
From: Venkatesh Pallipadi <venkatesh.pallipadi@intel.com>

The specifications that talk about E820 map doesn't have an upper limit on
the number of e820 entries.  But, today's kernel has a hard limit of 32. 
With increase in memory size, we are seeing the number of E820 entries
reaching close to 32.  Patch below bumps the number upto 128.  

The patch changes the location of EDDBUF in zero-page (as it comes after E820).
As, EDDBUF is not used by boot loaders, this patch should not have any effect
on bootloader-setup code interface.

Patch covers both i386 and x86-64.

Tested on:
* grub booting bzImage
* lilo booting bzImage with EDID info enabled
* pxeboot of bzImage

Side-effect: 
bss increases by ~ 2K and init.data increases by ~7.5K
on all systems, due to increase in size of static arrays.

Signed-off-by: Venkatesh Pallipadi <venkatesh.pallipadi@intel.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 Documentation/i386/zero-page.txt |    6 +++---
 arch/i386/boot/setup.S           |    6 +++---
 arch/x86_64/boot/setup.S         |    6 +++---
 arch/x86_64/kernel/head64.c      |    4 +---
 arch/x86_64/kernel/setup64.c     |    3 ++-
 include/asm-i386/e820.h          |    2 +-
 include/asm-i386/setup.h         |    2 +-
 include/asm-x86_64/bootsetup.h   |    3 ++-
 include/asm-x86_64/e820.h        |    2 +-
 include/linux/edd.h              |    2 +-
 10 files changed, 18 insertions(+), 18 deletions(-)

diff -puN arch/i386/boot/setup.S~increase-number-of-e820-entries-hard-limit-from-32-to-128 arch/i386/boot/setup.S
--- 25/arch/i386/boot/setup.S~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/arch/i386/boot/setup.S	Thu Apr 28 17:08:35 2005
@@ -164,7 +164,7 @@ ramdisk_max:	.long (-__PAGE_OFFSET-(512 
 trampoline:	call	start_of_setup
 		.align 16
 					# The offset at this point is 0x240
-		.space	(0x7ff-0x240+1) # E820 & EDD space (ending at 0x7ff)
+		.space	(0xeff-0x240+1) # E820 & EDD space (ending at 0xeff)
 # End of setup header #####################################################
 
 start_of_setup:
@@ -333,9 +333,9 @@ jmpe820:
 	# sizeof(e820rec).
 	#
 good820:
-	movb	(E820NR), %al			# up to 32 entries
+	movb	(E820NR), %al			# up to 128 entries
 	cmpb	$E820MAX, %al
-	jnl	bail820
+	jae	bail820
 
 	incb	(E820NR)
 	movw	%di, %ax
diff -puN arch/x86_64/boot/setup.S~increase-number-of-e820-entries-hard-limit-from-32-to-128 arch/x86_64/boot/setup.S
--- 25/arch/x86_64/boot/setup.S~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/arch/x86_64/boot/setup.S	Thu Apr 28 17:08:35 2005
@@ -160,7 +160,7 @@ ramdisk_max:	.long 0xffffffff
 trampoline:	call	start_of_setup
 		.align 16
 					# The offset at this point is 0x240
-		.space  (0x7ff-0x240+1)	# E820 & EDD space (ending at 0x7ff)
+		.space  (0xeff-0x240+1)	# E820 & EDD space (ending at 0xeff)
 # End of setup header #####################################################
 
 start_of_setup:
@@ -412,9 +412,9 @@ jmpe820:
 	# sizeof(e820rec).
 	#
 good820:
-	movb	(E820NR), %al			# up to 32 entries
+	movb	(E820NR), %al			# up to 128 entries
 	cmpb	$E820MAX, %al
-	jnl	bail820
+	jae	bail820
 
 	incb	(E820NR)
 	movw	%di, %ax
diff -puN arch/x86_64/kernel/head64.c~increase-number-of-e820-entries-hard-limit-from-32-to-128 arch/x86_64/kernel/head64.c
--- 25/arch/x86_64/kernel/head64.c~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/arch/x86_64/kernel/head64.c	Thu Apr 28 17:08:35 2005
@@ -29,8 +29,6 @@ static void __init clear_bss(void)
 	       (unsigned long) __bss_end - (unsigned long) __bss_start);
 }
 
-extern char x86_boot_params[2048];
-
 #define NEW_CL_POINTER		0x228	/* Relative to real mode data */
 #define OLD_CL_MAGIC_ADDR	0x90020
 #define OLD_CL_MAGIC            0xA33F
@@ -44,7 +42,7 @@ static void __init copy_bootdata(char *r
 	int new_data;
 	char * command_line;
 
-	memcpy(x86_boot_params, real_mode_data, 2048); 
+	memcpy(x86_boot_params, real_mode_data, BOOT_PARAM_SIZE);
 	new_data = *(int *) (x86_boot_params + NEW_CL_POINTER);
 	if (!new_data) {
 		if (OLD_CL_MAGIC != * (u16 *) OLD_CL_MAGIC_ADDR) {
diff -puN arch/x86_64/kernel/setup64.c~increase-number-of-e820-entries-hard-limit-from-32-to-128 arch/x86_64/kernel/setup64.c
--- 25/arch/x86_64/kernel/setup64.c~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/arch/x86_64/kernel/setup64.c	Thu Apr 28 17:08:35 2005
@@ -12,6 +12,7 @@
 #include <linux/string.h>
 #include <linux/bootmem.h>
 #include <linux/bitops.h>
+#include <asm/bootsetup.h>
 #include <asm/pda.h>
 #include <asm/pgtable.h>
 #include <asm/processor.h>
@@ -26,7 +27,7 @@
 #include <asm/mman.h>
 #include <asm/numa.h>
 
-char x86_boot_params[2048] __initdata = {0,};
+char x86_boot_params[BOOT_PARAM_SIZE] __initdata = {0,};
 
 cpumask_t cpu_initialized __initdata = CPU_MASK_NONE;
 
diff -puN Documentation/i386/zero-page.txt~increase-number-of-e820-entries-hard-limit-from-32-to-128 Documentation/i386/zero-page.txt
--- 25/Documentation/i386/zero-page.txt~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/Documentation/i386/zero-page.txt	Thu Apr 28 17:08:35 2005
@@ -79,6 +79,6 @@ Offset	Type		Description
 0x22c   unsigned long	ramdisk_max
 0x230   16 bytes 	trampoline
 0x290 - 0x2cf		EDD_MBR_SIG_BUFFER (edd.S)
-0x2d0 - 0x600		E820MAP
-0x600 - 0x7ff		EDDBUF (edd.S) for disk signature read sector
-0x600 - 0x7eb		EDDBUF (edd.S) for edd data
+0x2d0 - 0xd00		E820MAP
+0xd00 - 0xeff		EDDBUF (edd.S) for disk signature read sector
+0xd00 - 0xeeb		EDDBUF (edd.S) for edd data
diff -puN include/asm-i386/e820.h~increase-number-of-e820-entries-hard-limit-from-32-to-128 include/asm-i386/e820.h
--- 25/include/asm-i386/e820.h~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/include/asm-i386/e820.h	Thu Apr 28 17:08:35 2005
@@ -13,7 +13,7 @@
 #define __E820_HEADER
 
 #define E820MAP	0x2d0		/* our map */
-#define E820MAX	32		/* number of entries in E820MAP */
+#define E820MAX	128		/* number of entries in E820MAP */
 #define E820NR	0x1e8		/* # entries in E820MAP */
 
 #define E820_RAM	1
diff -puN include/asm-i386/setup.h~increase-number-of-e820-entries-hard-limit-from-32-to-128 include/asm-i386/setup.h
--- 25/include/asm-i386/setup.h~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/include/asm-i386/setup.h	Thu Apr 28 17:08:35 2005
@@ -16,7 +16,7 @@
 #define MAXMEM_PFN	PFN_DOWN(MAXMEM)
 #define MAX_NONPAE_PFN	(1 << 20)
 
-#define PARAM_SIZE 2048
+#define PARAM_SIZE 4096
 #define COMMAND_LINE_SIZE 256
 
 #define OLD_CL_MAGIC_ADDR	0x90020
diff -puN include/asm-x86_64/bootsetup.h~increase-number-of-e820-entries-hard-limit-from-32-to-128 include/asm-x86_64/bootsetup.h
--- 25/include/asm-x86_64/bootsetup.h~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/include/asm-x86_64/bootsetup.h	Thu Apr 28 17:08:35 2005
@@ -2,7 +2,8 @@
 #ifndef _X86_64_BOOTSETUP_H
 #define _X86_64_BOOTSETUP_H 1
 
-extern char x86_boot_params[2048];
+#define BOOT_PARAM_SIZE		4096
+extern char x86_boot_params[BOOT_PARAM_SIZE];
 
 /*
  * This is set up by the setup-routine at boot-time
diff -puN include/asm-x86_64/e820.h~increase-number-of-e820-entries-hard-limit-from-32-to-128 include/asm-x86_64/e820.h
--- 25/include/asm-x86_64/e820.h~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/include/asm-x86_64/e820.h	Thu Apr 28 17:08:35 2005
@@ -14,7 +14,7 @@
 #include <linux/mmzone.h>
 
 #define E820MAP	0x2d0		/* our map */
-#define E820MAX	32		/* number of entries in E820MAP */
+#define E820MAX	128		/* number of entries in E820MAP */
 #define E820NR	0x1e8		/* # entries in E820MAP */
 
 #define E820_RAM	1
diff -puN include/linux/edd.h~increase-number-of-e820-entries-hard-limit-from-32-to-128 include/linux/edd.h
--- 25/include/linux/edd.h~increase-number-of-e820-entries-hard-limit-from-32-to-128	Thu Apr 28 17:08:35 2005
+++ 25-akpm/include/linux/edd.h	Thu Apr 28 17:08:35 2005
@@ -32,7 +32,7 @@
 
 #define EDDNR 0x1e9		/* addr of number of edd_info structs at EDDBUF
 				   in boot_params - treat this as 1 byte  */
-#define EDDBUF	0x600		/* addr of edd_info structs in boot_params */
+#define EDDBUF	0xd00		/* addr of edd_info structs in boot_params */
 #define EDDMAXNR 6		/* number of edd_info structs starting at EDDBUF  */
 #define EDDEXTSIZE 8		/* change these if you muck with the structures */
 #define EDDPARMSIZE 74
_
